/* ----------------------------------------------------------------------
   (C) Copyright Marxmeier Software AG, 2002-2012

   File:     eloqdb.h
   Function: Describes the public Eloquence database api
   @(#)$Revision: 27.4 $

   Edit History:
   04.06.2004 - added function ids and S_TRHEAD status code
   24.11.2005 - added idb_memo()
   26.04.2007 - remove extra type definitions
              - idb_rollback() uses int instead of u_int32_t
   21.11.2012 - added struct info_506, info_512, info_521, info_602
              - added idb_audit()
              - added IDBCAP_FTS, IDBCAP_FTS_COMPAT, IDBCAP_FTS_EXT

---------------------------------------------------------------------- */

#ifndef ELOQDB_INCLUDED
#define ELOQDB_INCLUDED

#include "sys/types.h"

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------
      idb limits
------------------------------------------------------------------------*/

/* limits */

#define IDB_MAX_ITEM_CNT  2048	/* max. number of data items */
#define IDB_MAX_ITEM_NAME 16	/* max. item name length */
#define IDB_MAX_SET_CNT   500   /* max. number of data sets */
#define IDB_MAX_SET_NAME  16	/* max. data set name length */
#define IDB_MAX_PATH_CNT  64	/* max. number of paths (hard limit) */
#define IDB_MAX_BUFFER    5120	/* max. user buffer size in bytes */
#define IDB_MAX_MEDIA_BUF 8000  /* max. media size of record (hard limit) */
#define IDB_MAX_LOCK_DESC 8192  /* max. size of a lock desc */
#define IDB_MAX_KEYSZ     512   /* max. key size */
#define IDB_MAX_KEYSEG    8     /* max. number of key segments (hard limit) */
#define IDB_MAX_GROUPS    8     /* max. number of access groups for a user */
#define IDB_MAX_TXNEST    20	/* max. transaction nesting */

/* backwards compatibility */

#define MAX_ITEM_CNT  IDB_MAX_ITEM_CNT
#define MAX_ITEM_NAME IDB_MAX_ITEM_NAME
#define MAX_SET_CNT   IDB_MAX_SET_CNT
#define MAX_SET_NAME  IDB_MAX_SET_NAME
#define MAX_PATH_CNT  IDB_MAX_PATH_CNT
#define DEF_MAX_BUF   IDB_MAX_BUFFER


/*------------------------------------------------------------------------
      idb info response
------------------------------------------------------------------------*/

struct info_list {         /* 103, 104, 203, 204 */
   int count;              /* list entries   */
   int no[1];              /* list data      */
};

struct info_102 {
   char name[16];          /* item name      */
   char type;              /* item type      */
   char filler[3];         /* gap (space)    */
   int size;               /* item size      */
   int count;              /* subitem count  */
   int fmt;                /* format number  */
};

struct info_202 {
   char name[16];          /* set name       */
   char type;              /* set type       */
   char filler[3];         /* gap (space)    */
   int rec_len;            /* record length  */
   int entries;            /* set entries    */
   int capacity;           /* set capacity   */
};

struct info_301 {
   int count;           /* list entries   */
   struct {
      int set;          /* related set    */
      int ditem;        /* detail item no */
   } path[1];
};

struct info_303 {
   int count;           /* list entries   */
   struct {
      int set;          /* related set    */
      int ditem;        /* detail item no */
      int sort_item;    /* sort item or 0 */
   } path[1];
};

struct info_302 {
   int item;            /* item no        */
   int set;             /* related set/0  */
};

struct info_502 {
   char name[16];        /* iitem name     */
   int  seg_cnt;         /* segment cnt    */
   struct {
      int item;          /* item no        */
      int length;        /* seg length     */
   } seg[8];
};

struct info_506 {
   char name[16];        /* iitem name     */
   int  flags;           /* iitem flags    */
   int  seg_cnt;         /* segment cnt    */
   struct {
      int item;          /* item no        */
      int start;         /* ofs into item  */
      int length;        /* seg length     */
      int resvd;         /* reserved       */
   } seg[8];
};

struct info_512 {
   char name[16];        /* fts fld name   */
   int  options;
   int  group;
   int  maxlen;
   int  minlen;
   int  excl;
   int  pcfg;
   int  synm;
   int  seg_cnt;         /* segment cnt    */
   int  resvd;           /* reserved       */
   struct {
      int item;          /* item no        */
      int start;         /* ofs into item  */
      int length;        /* seg length     */
      int resvd;         /* reserved       */
   } seg[8];
};

struct info_521 {
   int set;
   int mset;
   int ditem;
   int res;
   int cnt;
   struct {
      int item;
      int group;
   } fld[1];
};

struct info_602 {
   int flags;            /* property type  */
   char value[1];        /* property value */
};

union info
{
   int no;              /* used by modes 101,201 */
   struct info_list list;
   struct info_102 item;
   struct info_202 set;
   struct info_301 path_list;
   struct info_302 search_item;
   struct info_303 path_list_303;
   struct info_502 iitem;
   struct info_506 iitem_506;
   struct info_512 fts_fld;
   struct info_521 fts_grp;
   struct info_602 property;
};


/*------------------------------------------------------------------------
      dbml function identifiers
------------------------------------------------------------------------*/

#define DB_OPEN     401
#define DB_INFO     402
#define DB_CLOSE    403
#define DB_FIND     404
#define DB_GET      405
#define DB_UPDATE   406
#define DB_PUT      407
#define DB_DELETE   408
#define DB_LOCK     409
#define DB_UNLOCK   410

#define DB_CREATE   411
#define DB_ERASE    412
#define DB_PURGE    413

#define DB_LOGON    420
#define DB_BEGIN    421
#define DB_ROLLBK   422
#define DB_COMMIT   423
#define DB_AUDIT    425
#define DB_MEMO     426

#define DB_SEQ_SCAN 430

#define DB_STORE    440
#define DB_RESTORE  441

#define DB_FTCFIND  450
#define DB_FTCGET   451
#define DB_FTCGETW  452


/*------------------------------------------------------------------------
      idb status codes
------------------------------------------------------------------------*/

#ifdef S_OK
#undef S_OK
#endif

#define S_OK        0      /* No error */
#define S_CANTOPEN -1      /* Unable to open database */
#define S_INUSE    -2      /* Database in use */
#define S_BADUSER  -3      /* User name not recognized */
#define S_BADPSWD  -4      /* User password does not match */
#define S_DENIED   -5      /* Permission denied */
#define S_NOSLOT   -10     /* No empty database slot */
#define S_BADID    -11     /* Bad database id */
#define S_NOLOCK   -12     /* Conflicting lock request on write attempt */
#define S_READONLY -14     /* Database opened in readonly mode */
#define S_BAD      -21     /* Bad parameter value */
#define S_NOWRITE  -23     /* No write access to data set */
#define S_BADAUTO  -24     /* Bad write attempt on automatic master */
#define S_BADMODE  -31     /* Bad mode */
#define S_DEADLK   -35     /* A deadlock condition has been detected */
#define S_BADITEM  -52     /* Item not an accessible search item */
#define S_BADLIST  -52     /* Bad list variable */
#define S_BADKEY   -54     /* Invalid key argument (syntax error) */
#define S_NORESULT -55     /* No search result */
#define S_RESTRUCT -80     /* Restructuring not possible */
#define S_VERSION  -91     /* Incompatible database version */
#define S_NOCREAT  -92     /* Database requires creation */
#define S_CORRUPT  -94     /* Database may be corrupt (obsolete) */
#define S_NOMASTER -95     /* No master set entry for current detail */
#define S_POINTER  -96     /* Corrupt pointer value detected */

#define S_NOMEM    -120    /* Not enough memory to perform DBLOCK */
#define S_LCKM122  -122    /* Descriptor list bad */
#define S_LCKM123  -123    /* Bad relational operator */
#define S_LCKM124  -124    /* Descriptor too short */
#define S_LCKM125  -125    /* Bad set name/number */
#define S_LCKM126  -126    /* Bad item name/numner */
#define S_LCKM127  -127    /* Attempt to lock using a compound item */
#define S_LCKM128  -128    /* Bad descriptor length for numeric item */
#define S_LCKM134  -134    /* Two descriptors conflict */
#define S_LCKM135  -135    /* Second lock not with wait option */
#define S_LCKM136  -136    /* descriptor length exceeds maximum */
#define S_LCKM137  -137    /* qualifier parameter of wrong type */

#define S_EOF       11     /* End of file reached */
#define S_DIRBOF    12     /* Directed beginning of file */
#define S_DIREOF    13     /* Directed end of file */
#define S_ENDCHAIN  15     /* End of chain reached */
#define S_NOREC     17     /* No current record or record/key not found */
#define S_BROKEN    18     /* Broken chain (only get!) */

#define S_LCK20     20     /* Database locked or contains locks */
#define S_LCK22     22     /* Dataset locked */
#define S_LCK23     23     /* Entries locked within set */
#define S_LCK24     24     /* Item conflicts with current locks */
#define S_LCK25     25     /* Entry or entries already locked */
#define S_LCKOP     27     /* Relational operator type conflict */

#define S_TXLEVEL   32     /* transaction nesting exceeds maximum */
#define S_NOTRANS   33     /* no transaction active */
#define S_TXSYNC    34     /* server transaction level out of sync */
#define S_TIMEOUT   36     /* Timeout occured during execution */

#define S_UPDATE    41     /* Cant alter search item */
#define S_DUPL      43     /* Duplicate key value */
#define S_CANTDEL   44     /* Can't delete master with non-empty chains */
#define S_BUFSPC    50     /* insufficient user buffer (eloqcore only) */
#define S_ARGTYPE   53     /* Bad argument type */
#define S_NOTFOUND  80     /* Set not created/not online */
#define S_COROPEN   94     /* Corrupt data base opened in mode 8 (obsolete) */
#define S_NOHEAD    100    /* No chain head for path xx */

#define S_ISAM      -800   /* Internal isam error (obsolete) */
#define S_DAEMON    -900   /* Daemon error (obsolete) */

#define S_VOLUME    -801   /* Volume failure */
#define S_VOLNODE   -802   /* Node related failure */
#define S_VOLFREC   -803   /* FixRec related problem */
#define S_BTREE     -804   /* BTREE related problem */
#define S_SYSCAT    -805   /* SysCat related problem */
#define S_SYSTEM    -806   /* System call */
#define S_LOCKDB    -807   /* locking failure */
#define S_MGMT      -808   /* management api failure */
#define S_IMAGE     -809   /* internal image failure */
#define S_REPL      -810   /* replication api failure */
#define S_SECAPI    -811   /* SECURE API failure */
#define S_ENCRYPTED -812   /* Encrypted database not available */
#define S_FTS       -813   /* FTS operation failed */


/*------------------------------------------------------------------------
   Remote access supplies its own status codes
   If a call fails, S_REMOTE is returned in status[0] and
   status[9] will either contain errno (if above zero) or one of the
   codes below.
------------------------------------------------------------------------*/

#define S_REMOTE -700      /* remote problem */

#define S_CONNECT   -2     /* unable to connect server */
#define S_INIT      -3     /* init failed */
#define S_ARGOVFL   -4     /* client arg buffer overflow */
#define S_SEND      -5     /* problem during send */
#define S_RECEIVE   -6     /* problem during receive */
#define S_ROOT      -7     /* inconsistent ROOT structures detected */
#define S_BUFFER    -8     /* inconsistent buffer size detected */
#define S_SERVERID  -9     /* unknown server identifier */
#define S_INCAPABLE -10    /* server does not have required capabilities */
#define S_THREAD    -11    /* thread lock/unlock failed */


/*------------------------------------------------------------------------
      prototypes
------------------------------------------------------------------------*/

#ifdef __hpux

/*
   The K&R compiler does not support prototypes. While everyone should
   use the ANSI compiler for anything serious, it may not be available
   because it is sold extra. So we make prototypes optional for now.
*/
#ifdef __cplusplus
#ifndef ELOQDB_PROTOTYPES
#define ELOQDB_PROTOTYPES
#endif
#else
#ifdef __STDC__
#ifndef ELOQDB_PROTOTYPES
#define ELOQDB_PROTOTYPES
#endif
#endif

#ifndef ELOQDB_PROTOTYPES
#ifndef size_t
#define size_t unsigned int
#define const /**/
#endif
#endif

#endif

#else

/*
   Here goes everything that's not HP-UX. Since K&R is a HP-UX only
   problem and everyone would use a ANSI compliant compiler anyway,
   we simply enable function prototypes.
*/
#define ELOQDB_PROTOTYPES

#endif


/*
   StatusType used to be int on HP-UX and long on WIN16 platform
   some ages ago. To preserve compatibility with existing source 
   code, we currently don't make this a typedef.
*/

#ifndef StatusType
#define StatusType int
#endif


/*------------------------------------------------------------------------
      Externals
------------------------------------------------------------------------*/

extern StatusType idb_status;    /* Status of last idb call */
extern StatusType idb_status2;   /* Secondary status */


/* ----------------------------------------------------------------------
      idb_capabilities() capability codes
---------------------------------------------------------------------- */

#define IDBCAP_IPC          1
#define IDBCAP_TRANSACTIONS 2
#define IDBCAP_MGMT         3
#define IDBCAP_LOGON        4
#define IDBCAP_SCAN         5
#define IDBCAP_CTL          6
#define IDBCAP_HP3K         7
#define IDBCAP_AUDIT        8
#define IDBCAP_REPLICATION  9
#define IDBCAP_FTS         11
#define IDBCAP_FTS_EXT     12
#define IDBCAP_FTS_COMPAT  10


/* ----------------------------------------------------------------------
      idb_ctl() commands
---------------------------------------------------------------------- */

#define IDBCTL_DCSETID 1
#define IDBCTL_SCSETID 2
#define IDBCTL_GCSETID 3


/*------------------------------------------------------------------------
      prototypes
------------------------------------------------------------------------*/

#ifdef ELOQDB_PROTOTYPES
#define _p(a) a
#else
#define _p(a) ()
#endif

/*
   Compatiblity stuff for Windows API:
*/

#ifdef _WIN32
#define DBCALL _stdcall
#endif

#ifndef DBCALL
#define DBCALL /**/
#endif

/* classic API */

int DBCALL idb_init _p((int, int, int, int));
int DBCALL idb_exit _p((void));
int DBCALL idb_open _p((const char *, const char *, int, StatusType []));
int DBCALL idb_close _p((int, const void *, int, StatusType []));
int DBCALL idb_info _p((int, const void *, int, StatusType [], void *));
int DBCALL idb_lock _p((int, const void *, int, StatusType []));
int DBCALL idb_unlock _p((int, const void *, int, StatusType []));
int DBCALL idb_find _p((int, const void *, int, StatusType [],
                    const void *, const void *));
int DBCALL idb_get _p((int, const void *, int, StatusType [],
               const void *, void *, const void *));
int DBCALL idb_put _p((int, const void *, int, StatusType [],
                   const void *, const void *));
int DBCALL idb_update _p((int, const void *, int, StatusType [],
                      const void *, const void *));
int DBCALL idb_delete _p((int, const void *, int, StatusType []));
int DBCALL idb_error _p((StatusType [], void *, int *));

/* extended API */

int DBCALL idbn_info _p((int dbid, const void *qual, int mode,
                     StatusType status[], void *infop, size_t infosz));
int DBCALL idbn_get _p(( int dbid, const void *qset, int mode,
                    StatusType status[], const void *lst,
                    void *buf, size_t bufsz, const void *arg ));
int DBCALL idbn_put _p(( int dbid, const void *qset, int mode,
                    StatusType status[],
                    const void *lst, const void *buf, size_t bufsz ));
int DBCALL idbn_update _p(( int dbid, const void *qset, int mode,
                       StatusType status[],
                       const void *lst, const void *buf, size_t bufsz ));
int DBCALL idbn_error _p((StatusType [], void *, size_t, int *));

/* mode 13 API */

int DBCALL idb_create _p((int dbid, int mode, StatusType status[],
                          int count, int list[]));
int DBCALL idb_erase _p((int dbid, int mode, StatusType status[],
                          int count, int list[]));
int DBCALL idb_purge _p((int dbid, int mode, StatusType status[],
                          int count, int list[]));

/* miscellaneous */

int DBCALL idb_capabilities _p((int, int));
int DBCALL idb_server_protocol_rev _p((int, int *, int *));
int DBCALL idb_ctl _p((int, int));
int DBCALL idb_logon _p((const char *user, const char *pswd));
int DBCALL idb_begin _p((const char *comment, int mode, StatusType status[]));
int DBCALL idb_rollback _p((int id, int mode, StatusType status[]));
int DBCALL idb_commit _p((int mode, StatusType status[]));

/* restructuring / maintenance */

int DBCALL idb__restruct_database _p((int dbid, StatusType status[]));

/* audit */

int DBCALL idb_audit _p((int mode, StatusType status[], 
                         int len, void *buf));

int DBCALL idb_memo _p((int mode, StatusType status[], 
                        int size, void *data));

#undef _p

#ifdef __cplusplus
}
#endif

#endif   /* ~ELOQDB_INCLUDED */
