/* -----------------------------------------------------------------------

   (C) Copyright Marxmeier Software AG 2012
   All rights reserved.

   File:     ftc.h
   Function: DBI/ODX API
   $Id: ftc.h,v 27.2 2013/01/30 14:08:20 mike Exp $
   
   This file provides the DBI/ODX call prototypes.

   Edit history:
   26.10.2012 - B.08.20 release

----------------------------------------------------------------------- */

#ifndef FTC_H
#define FTC_H

/* include image3k defitions */
#include "image3k.h"

#if defined(__cplusplus)
extern "C" {
#endif


/*
   ODX calls
*/

enum odx_Intrinsic_ID {
   ODXFIND_INTRINSIC_ID     = 801, /* 2xx */
   ODXGET_INTRINSIC_ID      = 802, /* 3xx */
   ODXGETWORD_INTRINSIC_ID  = 803, /* 3xx */
   ODXINFO_INTRINSIC_ID     = 804, /* 5xx */
   ODXPRINT_INTRINSIC_ID    = 805, /* 8xx */
   ODXTRANSFER_INTRINSIC_ID = 806, /* 8xx */
   ODXVIEW_INTRINSIC_ID     = 807  /* 8xx */
};

/* entry points */

extern void DBCALL3K odxfind (
   const void *base, const void *dset, const void *modep,
   void *status, const void *field, const void *keywords);

extern void DBCALL3K odxget (
   const void *base, const void *modep,
   void *status, void *si_list, const void *si_countp);

extern void DBCALL3K odxgetword (
   const void *base, const void *modep,
   void *status, void *target);

extern void DBCALL3K odxinfo (
   const void *base, const void *qual, const void *modep,
   void *status, void *buf);

extern void DBCALL3K odxtransfer(
   const void *base, const void *modep,
   void *status, void *filename, void *options);

extern void DBCALL3K odxprint(
   const void *filename, const void *keywords,
   const void *control, void *status, void *plabels);

extern void DBCALL3K odxview(
   const void *filename, const void *keywords,
   const void *control, void *status, void *plabels);

extern void DBCALL3K odxfind (
   const void *base, const void *dset, const void *modep,
   void *status, const void *field, const void *keywords);

/* internal implementation */

extern void DBCALL3K _odxget (
   const void *base, const void *modep,
   void *status, void *si_list, const void *si_countp);

extern void DBCALL3K _odxgetword (
   const void *base, const void *modep,
   void *status, void *target);

extern void DBCALL3K _odxinfo (
   const void *base, const void *qual, const void *modep,
   void *status, void *buf);

extern void DBCALL3K _odxtransfer(
   const void *base, const void *modep,
   void *status, void *filename, void *options);

extern void DBCALL3K _odxprint(
   const void *filename, const void *keywords,
   const void *control, void *status, void *plabels);

extern void DBCALL3K _odxview(
   const void *filename, const void *keywords,
   const void *control, void *status, void *plabels);


/* 
   DBI calls
*/

enum dbi_Intrinsic_ID {
   DBIPUT_INTRINSIC_ID      = 901, /* 1xx */
   DBIFIND_INTRINSIC_ID     = 902, /* 2xx */
   DBIGET_INTRINSIC_ID      = 903, /* 3xx */
   DBIDELETE_INTRINSIC_ID   = 904, /* 4xx */
   DBIINFO_INTRINSIC_ID     = 905, /* 5xx */
   DBIUPDATE_INTRINSIC_ID   = 906, /* 6xx */
   DBIOPEN_INTRINSIC_ID     = 907, /* 9xx */
   DBICLOSE_INTRINSIC_ID    = 908,
   DBILOCK_INTRINSIC_ID     = 909,
   DBIUNLOCK_INTRINSIC_ID   = 910
};

/* entry points */

extern void DBCALL3K dbiclose(
   void *base, const void *dset, 
   const void *modep, void *status);

extern void DBCALL3K dbidelete(
   const void *base, const void *dset, 
   const void *modep, void *status);

extern void DBCALL3K dbierror(
   const void *status, void *buffer, void *textlenp);

extern void DBCALL3K dbiexplain(
   const void *status, const void *parm);

extern void DBCALL3K dbifind(
   const void *base, const void *dset, const void *modep,
   void *status, const void *item, const void *argument);

extern void DBCALL3K dbiget(
   const void *base, const void *dset, const void *modep,
   void *status, const void *list, void *buffer,
   const void *argument);

extern void DBCALL3K dbiinfo(
   const void *base, const void *qualifier, 
   const void *modep, void *status, void *buffer);

extern void DBCALL3K dbilock(
   const void *base, const void *qualifier, 
   const void *modep, void *status);

extern void DBCALL3K dbiopen(
   void *base, const void *pswd, 
   const void *modep, void *status);

extern void DBCALL3K dbiput(
   const void *base, const void *dset, const void *modep,
   void *status, const void *list, const void *buffer);

extern void DBCALL3K dbiunlock(
   const void *base, const void *qualifier, 
   const void *modep, void *status);

extern void DBCALL3K dbiupdate(
   const void *base, const void *dset, const void *modep,
   void *status, const void *list, const void *buffer);

/* internal implementation */

extern void DBCALL3K _dbiclose(
   void *base, const void *dset, 
   const void *modep, void *status);

extern void DBCALL3K _dbidelete(
   const void *base, const void *dset, 
   const void *modep, void *status);

extern void DBCALL3K _dbierror(
   const void *status, void *buffer, void *textlenp);

extern void DBCALL3K _dbiexplain(
   const void *status, const void *parm);

extern void DBCALL3K _dbifind(
   const void *base, const void *dset, const void *modep,
   void *status, const void *item, const void *argument);

extern void DBCALL3K _dbiget(
   const void *base, const void *dset, const void *modep,
   void *status, const void *list, void *buffer,
   const void *argument);

extern void DBCALL3K _dbiinfo(
   const void *base, const void *qualifier, 
   const void *modep, void *status, void *buffer);

extern void DBCALL3K _dbilock(
   const void *base, const void *qualifier, 
   const void *modep, void *status);

extern void DBCALL3K _dbiopen(
   void *base, const void *pswd, 
   const void *modep, void *status);

extern void DBCALL3K _dbiput(
   const void *base, const void *dset, const void *modep,
   void *status, const void *list, const void *buffer);

extern void DBCALL3K _dbiunlock(
   const void *base, const void *qualifier, 
   const void *modep, void *status);

extern void DBCALL3K _dbiupdate(
   const void *base, const void *dset, const void *modep,
   void *status, const void *list, const void *buffer);

#if defined(__cplusplus)
}
#endif

#endif /* FTC_H */
